// console.cpp : Defines the entry point for the console application.
//

#include <windows.h>
#include <stdio.h>
#include <winbase.h>
#include <conio.h>

extern "C" {
  #include "usbi2cio.h"
}

HANDLE HandleInfoExample(void);
void GeneralPurposeIoExample(HANDLE hDevInstance);
void I2cExample(HANDLE hDevInstance);
void BulkTransferExample(HANDLE hDevInstance);


int main(int argc, char* argv[]) {

	HANDLE hDevInstance = INVALID_HANDLE_VALUE; // device instance handle

	printf("\nCalling Handle/Info Example function\n");
	hDevInstance = HandleInfoExample();

	if(hDevInstance != INVALID_HANDLE_VALUE) {
		// we have a valid handle to a device instance, ok to continue with other examples.

		// Call the IoExample function (see IoExample function definition below)
//		printf("\nCalling GeneralPurposeIoExample function\n");
//		GeneralPurposeIoExample(hDevInstance);

		// Call the I2cExample function (see I2cExample function definition below)
//		printf("\nCalling I2cExample function\n");
//		I2cExample(hDevInstance);

		// Call the BulkExample function (see BulkExample function definition below)
//		printf("\nCalling BulkTransferExample function\n");
//		BulkTransferExample(hDevInstance);

        // close the handle to device instance
		printf("\nClosing handle to board\n\n");
		DAPI_CloseDeviceInstance(hDevInstance);
		while (!kbhit());
		return 1;
	}
	else {
		// we were unable to open a handle to the USB-I2C/IO board
		printf("Critical error, without a valid device handle, no further testing is possible, exiting\n");
		while (!kbhit());
		return 0;
	}
}

HANDLE HandleInfoExample(void) {
/*
	This function will eventually perform all of the function calls provided by the Handle/Info group of the API.

	For more information on the following functions, please consult the "API User's Guide" document.

 	Version Checking
		GetDllVersion	Returns the DLL version information.
		GetDriverVersion	Returns the Device Driver version information.
		GetFirmwareVersion	Returns the Firmware version for the specified device.

	Discovery and Detection
		GetDeviceCount	Returns the number of attached devices.
		GetDeviceInfo	Generates a list of device information for attached devices.
		GetSerialId	Obtains a device's Serial ID string (unique identifier).
		DetectDevice	Return true if device handle is valid (device present).

	Device Handle
		OpenDeviceInstance	Opens a handle to a specified device instance.
		CloseDeviceInstance	Closes a device instance handle
		OpenDeviceBySerialId	Obtains a handle to a device specified by it Serial ID.
*/

	HANDLE hDevInstance = INVALID_HANDLE_VALUE; // device instance handle
  WORD wVersion, i;
	BYTE high, low;
	char SerialId[9] = "";

	// *************************************** Version Checking calls ********************************************
	// WORD _stdcall DAPI_GetDllVersion(void);
	wVersion = DAPI_GetDllVersion();
	high = (wVersion >> 8) & 0x00ff;
	low = wVersion & 0x00ff;
	printf(" DLL Version = %d.%d\n", high, low);

	// WORD _stdcall DAPI_GetDriverVersion(void);

//  printf("\nGetting driver version (OS)\n");
  wVersion = DAPI_GetDriverVersion();
  printf("\n DRIVER Version (via OS)= 0x%x\n",wVersion);

				// WORD _stdcall DAPI_GetFimwareVersion(HANDLE hDevInstance);
	// not implemented

	// *********************************** Discovery and Detection calls ****************************************

	// BYTE _stdcall DAPI_GetDeviceCount( LPSTR lpsDevName );
/*
	This section is commented out due to a printf in the dll that causes 126 lines of debug information to be
	generated (one line for each of the 127 possible devices instances that are not actually present).
	I will re-enable this section after I generate a fixed dll.
*/

	i = DAPI_GetDeviceCount("UsbI2cIo");
	printf(" Device Count = %d\n", i);

	// BYTE _stdcall DAPI_GetDeviceInfo( LPSTR lpsDevName, LPDEVINFO lpDevInfo);
	// test call not implemented... will add later.


	// these two calls are performed in the next section, as they require a valid device handle as a parameter
	// BOOL _stdcall DAPI_GetSerialId(HANDLE hDevInstance, LPSTR lpsSerialId);
	// BOOL _stdcall DAPI_DetectDevice(HANDLE hDevInstance);
	

	// ***************************************** Device Handle calls ********************************************

	// attempt to open device UsbI2cIo0 (instance 0 of the device with symbolic name UsbI2cIo)
	// HANDLE _stdcall DAPI_OpenDeviceInstance(LPSTR lpsDevName, BYTE byDevInstance); 
	hDevInstance = DAPI_OpenDeviceInstance("UsbI2cIo", 0);

	// check that we actually got a good handle
	if( hDevInstance != INVALID_HANDLE_VALUE) {
		// success - device instance handle opened
		printf(" Opened device UsbI2cIo0\n");

/*
//		printf("\nGetting driver version (Driver)\n");
		wVersion = DAPI_GetDriverVersionEx(hDevInstance);
		printf("\n DRIVER Version (via Driver) = 0x%x\n",wVersion);
*/

		// BOOL _stdcall DAPI_GetSerialId(HANDLE hDevInstance, LPSTR lpsSerialId);
		if(DAPI_GetSerialId(hDevInstance, SerialId)) {
			// read a serial number from board
			printf(" Serial Number = %s\n", SerialId);
		}
		else {
			printf(" Error: Unable to read Serial Number from board\n");
		}
			
		if(DAPI_GetFirmwareVersion(hDevInstance, &wVersion)) {
			// read firmware version from board
			high = (wVersion >> 8) & 0x00ff;
			low = wVersion & 0x00ff;
			printf(" FIRMWARE Version = %d.%d\n", high, low);
		}
		else {
			printf(" Error: Unable to read firmware version from board\n");
		}

		// BOOL _stdcall DAPI_DetectDevice(HANDLE hDevInstance);
		if(DAPI_DetectDevice(hDevInstance)) {
			// device was detected
			printf(" Device was detected\n");
		}
		else {
			printf(" Error: Device was not detected\n");
		}
/*

		This section is commented out due to a printf in the dll that causes 126 lines of debug information to be
		generated (one line for each of the 127 possible devices instances that are not actually present).
		I will re-enable this section after I generate a fixed dll.
*/

		// BOOL _stdcall DAPI_CloseDeviceInstance(HANDLE hDevInstance);
		DAPI_CloseDeviceInstance(hDevInstance);
		printf(" Closed handle to board\n");

		// open a handle to the same board using the serial number we previously obtained.
		// HANDLE _stdcall DAPI_OpenDeviceBySerialId(LPSTR lpsDevName, LPSTR lpsDevSerialId);
		hDevInstance = DAPI_OpenDeviceBySerialId("UsbI2cIo", SerialId);

		if( hDevInstance != INVALID_HANDLE_VALUE) {
			// success - device instance handle opened
			printf(" Opened handle to device UsbI2cIo, serial number %s\n", SerialId);
		}
		else {
			// failure - device instance handle not opened
			printf(" Error: Failed to open handle to device UsbI2cIo, serial number %s\n", SerialId);
		}
	}
	else {
		printf(" Error: Failed to open a handle to device UsbI2cIo0\n");
	}
	return hDevInstance;
}
	

void GeneralPurposeIoExample(HANDLE hDevInstance) {
		
	// This function demonstrates I/O functionality by configuring, writing and reading the I/O pins.

	// Please refer to the API User's Guide for detailed function calling information

	// Pins are bit configurable, any pin can be an input or an output
	// Pins default to inputs on power-up, power-up states can be set with external pull-up or pull-down resistors
		
	ULONG ulConfig;		      // variable used to hold pin configuration value read from board
	ULONG ulOutPins;        // variable used to hold I/O pin values output to the board
	ULONG ulPinsMask;				// variable used to specify which output pins should be modified
													// this extra "mask" parameter makes it unnecessary to perform a "read-modify-write"
													// operation in order to effect only the value of some bits, while leaving others
													// unchanged.
	ULONG ulInPins;	        // variable used to hold I/O pin values read from the board

		
	// configure all general purpose I/O port pins as as Outputs
	// BOOL _stdcall DAPI_ConfigIoPorts(HANDLE hDevInstance, ULONG ulIoPortConfig);
	DAPI_ConfigIoPorts(hDevInstance, 0x00000000); // bits that are: 0 = output, 1 = input
	printf(" Configured all general purpose I/O port pins as Outputs\n");

	// read the current configuration of the general purpose I/O port pins
	// BOOL _stdcall DAPI_GetIoConfig(HANDLE hDevInstance, LPLONG lpulIoPortConfig);
	DAPI_GetIoConfig(hDevInstance, (LPLONG)&ulConfig); // bits that are: 0 = output, 1 = input

	// mask off value of upper three nibbles, which are unused, and test value of lower 5 nibbles
	if( (ulConfig & 0x000FFFFF) == 0x00000000) {
		printf(" Read configuration value of 0x%08X from the board\n", ulConfig);
	}
	else {
		printf(" Error: Read configuration value of 0x%08X from the board\n", ulConfig);
	}

	// ***************************************** Set all pins to zero (lows) ***************************************
	printf(" \n Setting all I/O pins to zero\n");
	ulOutPins = 0;
	ulPinsMask = 0xFFFFFFFF;  // a mask value of 0xFFFFFFFF means all pins are modified (none are masked).

	// BOOL _stdcall DAPI_WriteIoPorts(HANDLE hDevInstance, ULONG ulIoPortData, ULONG ulIoPortMask);
	DAPI_WriteIoPorts(hDevInstance, ulOutPins, ulPinsMask);

	// ********************************** Write a walking bit pattern to the pins **********************************
	ulPinsMask = 0x00000001;	// setup for 

	printf(" \n Writing a \"Walking Bit\" pattern to the I/O pins\n");
	for(ulOutPins=0x00000001; ulOutPins < 0x00100000; ) {
		// BOOL _stdcall DAPI_WriteIoPorts(HANDLE hDevInstance, ULONG ulIoPortData, ULONG ulIoPortMask);
		DAPI_WriteIoPorts(hDevInstance, ulOutPins, ulPinsMask);
		printf(" Wrote ulOutPins of 0x%08X and ulPinsMask of 0x%08X\n", ulOutPins, ulPinsMask);
		DAPI_ReadIoPorts(hDevInstance, (LONG *)&ulInPins);
		if(ulInPins == ulOutPins) {
			printf(" Read value 0x%08X from the I/O pins\n", ulInPins);
		}
		else {
			printf(" Error: Read value 0x%08X from the I/O pins\n", ulInPins);
		}

		ulOutPins = ulOutPins << 1;			// left shift pins value to walk bit
		ulPinsMask = ulPinsMask << 1;		// left shift pins mask to walk which bits are effected
		if(ulPinsMask == 0x00000002) {
			ulPinsMask += 1;						  // add one to ulPinsMask value on the first pass, this makes two pins be
																		// effected on each pass, the pin being set on each pass, and the pin that
																		// was set on the previous pass (which will be cleared).
		}

	}
}

void I2cExample(HANDLE hDevInstance) {

	// This function demontrates I2C functionality by performing a series of writes and reads to/from
	// the onboard I2C eeprom.

	// The board must have jumpers on both the "WP" and "ENABLE" positions of JP2.

	I2C_TRANS I2cTrans;		// An instance of an I2C Transaction Structure, used for I2C operations
	LONG lRetVal;					// variable used to hold value returned from function calls
	int i;								// for loop indexing

	// Initialize the I2C structure for the type of I2C transaction we want to perform
	// Structure definition shown here for easy reference:

	/*
		typedef struct _I2C_TRANS {
			BYTE byTransType;
			BYTE bySlvDevAddr;
			WORD wMemoryAddr;
			WORD wCount;
			BYTE Data[256];
		} I2C_TRANS, *PI2C_TRANS;
	*/


	// The onboard eeprom uses special I2C protocol for memory devices with 2 byte addresses
	// this is an internal address within the devices memory array, not the I2C bus address.
	// So, to have the USB-I2C/IO automatically perform the special I2C protocol, we specify
	// a transaction type of I2C_TRANS_16ADR when communicating with the onboard eeprom.
	// Other type of I2C devices may use one of the other transaction types.
	// Transaction types shown here for easy reference:

	/*
		typedef enum {
			// supported transaction types
			I2C_TRANS_NOADR,          // read or write with no address cycle
			I2C_TRANS_8ADR,           // read or write with 8 bit address cycle
			I2C_TRANS_16ADR,          // read or write with 16 bit address cycle
			I2C_TRANS_NOADR_NS        // read or write with no address cycle, stop signaling suppressed
		} I2C_TRANS_TYPE;
	*/

	// **************************** Setup things that need to be setup once ****************************

	// Specify the I2C Transaction type for this transaction.
	I2cTrans.byTransType = I2C_TRANS_16ADR;

	// Specify the I2C bus address of the onboard eeprom
	I2cTrans.bySlvDevAddr = 0xA2;

	// Specify the address within the onboard eeprom memory array at which we will begin our access
	I2cTrans.wMemoryAddr = 0x3D00;

	// Specify the number of bytes we will read from the onboard I2C eeprom
	I2cTrans.wCount = 64;

	// ************** first we will zero the page of the eeprom that we will be testing ****************

	// so... we need to initialize our write data to zeroes
	for(i=0; i<64; i++) {
		I2cTrans.Data[i] = 0;
	}

	// now, we are ready to write to the eeprom.
	// Perform the I2C WRITE transaction
	// LONG _stdcall DAPI_WriteI2c(HANDLE hDevInstance, PI2C_TRANS TransI2C);
	lRetVal = DAPI_WriteI2c(hDevInstance, &I2cTrans);

	// check the return value to determine if transaction was successful
	if (lRetVal = I2cTrans.wCount) {
		// great, we wrote the same number of bytes that we requested to write

		// display the data we wrote to the eeprom (zeroes)
		printf(" Wrote 64 bytes of \"0\" data to I2C eeprom beginning at address 0x%04X\n", I2cTrans.wMemoryAddr);
	}
	else {
		// error, we didn't get what we asked for
		printf(" Error: DAPI_WriteI2c returned %d, expected %d\n", lRetVal, I2cTrans.wCount);
	}

	Sleep(10);	// because we wrote to an I2C eeprom, and are about to read it... we need a short pause to allow
							// the eeproms internal write cycle to complete before attempting the read.

	// ************** next we will verify that the eeprom page really got "zeroed" ****************

	// Perform an I2C READ transaction
	// LONG _stdcall DAPI_ReadI2c(HANDLE hDevInstance, PI2C_TRANS TransI2C);
	lRetVal = DAPI_ReadI2c(hDevInstance, &I2cTrans);

	// check the return value to determine if transaction was successful
	if (lRetVal == I2cTrans.wCount) {
		// great, we read the same number of bytes that we requested to read

		// test that the data read was all zeroes
		for(i=0; i<64; i++) {
			if(I2cTrans.Data[i] != 0) {
				printf(" Error: Read 0x%02X from location 0x%04X\n", I2cTrans.Data[i], I2cTrans.wMemoryAddr + i);
				break;	// stop on first detected error
			};
		}

		// test for success, and display success message
		if(i == 64) {
			// display the data we read from the eeprom (zeroes)
			printf(" Read 64 bytes of \"0\" data from I2C eeprom beginning at address 0x%04X\n", I2cTrans.wMemoryAddr);
		}
	}
	else {
		// error, we didn't get what we asked for
		printf(" Error: DAPI_ReadI2c returned %d, expected %d\n", lRetVal, I2cTrans.wCount);
	}

	// ***** next we will write an incrementing pattern to the page of the eeprom that we will be testing *****

	// so... we need to initialize our write data to an incrementing pattern
	for(i=0; i<64; i++) {
		I2cTrans.Data[i] = i;
	}

	// now, we are ready to write to the eeprom.
	// Perform the I2C WRITE transaction
	// LONG _stdcall DAPI_WriteI2c(HANDLE hDevInstance, PI2C_TRANS TransI2C);
	lRetVal = DAPI_WriteI2c(hDevInstance, &I2cTrans);

	// check the return value to determine if transaction was successful
	if (lRetVal == I2cTrans.wCount) {
		// great, we wrote the same number of bytes that we requested to write

		// display the data we wrote to the eeprom (zeroes)
		printf(" Wrote 64 bytes of incrementing data to I2C eeprom beginning at address 0x%04X\n");
	}
	else {
		// error, we didn't get what we asked for
		printf(" Error: DAPI_WriteI2c returned %d, expected %d\n", lRetVal, I2cTrans.wCount);
	}

	Sleep(10);	// because we wrote to an I2C eeprom, and are about to read it... we need a short pause to allow
							// the eeproms internal write cycle to complete before attempting the read.

	// ************** next we will verify that the eeprom page really got written ****************

	// Perform an I2C READ transaction
	// LONG _stdcall DAPI_ReadI2c(HANDLE hDevInstance, PI2C_TRANS TransI2C);
	lRetVal = DAPI_ReadI2c(hDevInstance, &I2cTrans);

	// check the return value to determine if transaction was successful
	if (lRetVal = I2cTrans.wCount) {
		// great, we read the same number of bytes that we requested to read

		// test that the data read was our incrementing pattern
		for(i=0; i<64; i++) {
			if(I2cTrans.Data[i] != i) {
				printf(" Error: Read 0x%02X from location 0x%04X\n", I2cTrans.Data[i], I2cTrans.wMemoryAddr + i);
				break;	// stop on first detected error
			};
		}

		// test for success, and display success message
		if(i == 64) {
			// display the data we read from the eeprom (zeroes)
			printf(" Read 64 bytes of incrementing data from I2C eeprom beginning at address 0x%04X\n", I2cTrans.wMemoryAddr);
		}
	}
	else {
		// error, we didn't get what we asked for
		printf(" Error: DAPI_ReadI2c returned %d, expected %d\n", lRetVal, I2cTrans.wCount);
	}

}

void BulkTransferExample(HANDLE hDevInstance) {
		
	// This function demonstrates Bulk transfer functionality by writing and reading via the bulk endpoints

	BOOL bRetVal;
	ULONG length = 2*64;
	BYTE * buffer = (BYTE *)malloc(length);
	ULONG i;

	if(buffer) {
		// allocated buffer, continue

		// initialize incrementing test data
		for (i = 0; i < length; i++)	{
			buffer[i] = (BYTE)(i & 0xFF); 
		}

		// Note that the direction bit in second parameter specifies whether it
		// is a OUT or IN bulk/interrupt transfer
		bRetVal = DAPI_TransferData(hDevInstance, 
                                0x02, //UCHAR Ep, 
                                buffer, 
                                &length);
		if(bRetVal) {
			// transferred data
			printf(" Bulk transferred %d bytes to device\n", length);
		}
		else {
			// failed to transfer data
			printf(" Error: bulk transfer of %d bytes failed\n", length);
		}

		free(buffer);		// free malloc'd memory
	}
	else {
		// failed to allocate buffer, display error message
		printf(" Error: malloc of %d bytes failed\n", length);
	}
}